#!/bin/bash
reset_ldap()
{
killall -9 slapd
for i in `seq 1 3` ; do
        if ! rm -f /rw2/var/lib/ldap/* ; then
		PROCESS=`lsof | grep /rw2/var/lib/ldap | head -n 1 | awk '{print $2}'`
		echo $PROCESS
		kill -9 $PROCESS
	fi
done

/usr/sbin/ldap_reset reset
}

dpkg_recover()
{
	#if [ "x$ans" = "x" ] ; then
	#	echo -ne "During this operation configuration might be lost. Do you want to proceed? (yes/No)"
	#        read ans
	#fi
	rm -rf /rw2/var/lib/dpkg
	tar xzf /rw2/var/backup/dpkg.tgz -C / &>/dev/null
	DPKG_RECOVERED=1
}

form_apt_conf()
{
	cat > /rw2/etc/apt.conf << EOF
DPkg
{
    // Old conffiles without prompt
    Options {"--force-confold";}
}
EOF
}

fix_wms_conf()
{
	if [ -z /rw2/etc/wms.conf ] ; then
	cat > /rw2/etc/wms.conf << EOF

[Settings]
basepath="/var/www/"
extension_rec=""

EOF
	fi
}

if ! mount -o remount,rw,async / ; then
	echo "Cannot re-mount root partition with rw option"
	exit 0 
fi
mount -o remount,rw,async /rw2

echo -ne "Fixing filesystem errors..."
#Init variables
tmpfile=`mktemp`
tmpfile2=`mktemp`
pkgs_file=`mktemp`
err_file=`mktemp`
ans=''
FAIL=0
DPKG_RECOVERED=0

#Fix dpkg database
ls -laR /var/lib/dpkg 2> $tmpfile > /dev/null
if [ "x`grep "Input/output error" $tmpfile`" != "x" ] ; then
        > $tmpfile
	for cnt in `seq 1 3` ; do
		dpkg_recover	
	done
fi
if [ ! -f /var/lib/dpkg/available ] ; then
	dpkg_recover
fi
if [ ! -f /var/lib/dpkg/diversions ] ; then
	dpkg_recover
fi
if [ ! -f /var/lib/dpkg/statoverride ] ; then
        dpkg_recover
fi
if [ ! -f /var/lib/dpkg/status ] ; then
        dpkg_recover
fi
if [ ! -d /var/lib/dpkg/alternatives ] ; then
        dpkg_recover
fi
if [ ! -d /var/lib/dpkg/info ] ; then
        dpkg_recover
fi
if [ ! -d /var/lib/dpkg/parts ] ; then
        dpkg_recover
fi
if [ ! -d /var/lib/dpkg/triggers ] ; then
        dpkg_recover
fi
if [ ! -d /var/lib/dpkg/updates ] ; then
        dpkg_recover
fi
if [ "x$DPKG_RECOVERED" != "x1" ] ; then
	for i in `dpkg --get-selections | grep install | awk '{print $1}'` ; do 
			dpkg -L $i &>/dev/null
			if [ "x$?" != "x0" ] ; then
				dpkg_recover
				break
			fi
	done
fi

#Fix wildix-con configuration
if [ "x`grep STARTUP /rw2/etc/pbx/net.conf`" = "x" ] ; then
#	if [ "x$ans" = "x" ] ; then
#	                echo -ne "During this operation configuration might be lost. Do you want to proceed? (yes/No)"
#	                read ans
#	fi
#	if [ "x$ans" = "xyes" ] ; then
		rm -f /rw2/etc/pbx/net.conf
		cat > /rw2/etc/pbx/net.conf <<EOF
STARTUP=0
SRV_IP=127.0.0.1
CLIENT_IP=5.15.0.1
PORT=1194
EOF
#	fi
fi

#Create new configuration file for apt
form_apt_conf

#If wms.conf has zero size - fill it in with default values
fix_wms_conf

#Fix init scripts permissions
chmod 0755 /etc/init.d/*

#Re-install slapd package
dpkg --configure -a &> /dev/null
apt-get install --reinstall -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf slapd &> /dev/null

#Always remove DB_CONFIG. It will be re-created on slapd start.
rm -f /rw2/var/lib/ldap/DB_CONFIG
#Fix LDAP configuration
slaptest -F /etc/ldap/slapd.d/ &>/dev/null
if [ "x$?" != "x0" ] ; then
	rm -rf /etc/ldap/*
	tar xzf /rw2/var/backup/slapd.tgz -C / &>/dev/null
fi
#Fix LDAP database
ls -laR /rw2/var/lib/ldap 2> $tmpfile > /dev/null
if [ "x`grep "Input/output error" $tmpfile`" != "x" ] ; then
	echo "Reseting LDAP db"
	reset_ldap
fi
ls -laR / 2> $tmpfile > /dev/null
if [ "x`grep "Input/output error" $tmpfile`" != "x" ] ; then
	grep "Input/output error" $tmpfile | awk '{print $4}' | awk -F ":" '{print $1}' > $tmpfile2
	> $tmpfile
	for FILENAME in `cat $tmpfile2` ; do
	        dpkg -S $FILENAME >> $pkgs_file 2>/dev/null
		if [ -d $FILENAME ] ; then
			for i in `seq 1 3` ; do
				if ! rm -rf $FILENAME ; then
					PROCESS=`lsof | grep "$FILENAME" | head -n 1 | awk '{print $2}'`
	                                kill -9 $PROCESS						
				fi
			done
		else
			for i in `seq 1 3` ; do
				if ! rm -f $FILENAME ; then
					PROCESS=lsof | grep "$FILENAME" | head -n 1 | awk '{print $2}'
	                                kill -9 $PROCESS
				fi
			done
		fi
	done
fi
rm -f $tmpfile2
echo "done."

echo -ne "Fixing dpkg database..."
/usr/bin/dpkg --configure --force-confold -a 2> /dev/null
if [ "x$?" != "x0" ] ; then
	echo "failed!"
	FAIL=1
fi
echo "OK"
#Check wmsdb. If it is corrupt try to recover from backup. If backup does not exist or all backups corrupt then we just remove it and it will be re-created by WMS.
echo -ne "Checking wmsdb..."
DBFILE="/rw2/var/www/wmsdb"
DBRECOVERED=0
if [ -f $DBFILE ] ; then
	if [ `sqlite3 $DBFILE '.ta' 2>/dev/null | wc -w` -lt 20 ] ; then
		echo -ne "corrupt. Trying to recover..."
		rm -f $DBFILE
		while [ "x$DBRECOVERED" = "x0" -a "x`ls -1t /rw2/var/www/wmsdb* 2>/dev/null | wc -l`" != "x0" ] ; do
			BACKUP=`ls -1t /rw2/var/www/wmsdb* 2>/dev/null | head -n 1`
			if [ `sqlite3 $BACKUP '.ta' 2>/dev/null | wc -w` -lt 20 ] ; then
				rm -f $BACKUP
			else
				cp $BACKUP $DBFILE
				chown wms.wms $DBFILE
				DBRECOVERED=1
			fi
		done
		if [ "x$DBRECOVERED" = "x1" ] ; then
			echo "recovered."
		elif [ "x$DBRECOVERED" = "x0" ] ; then
			echo "failed. Using default one."
		fi
	else
		echo "done."
	fi
else
	echo "not exist. Using default one."
fi

echo -ne "Re-installing basic packages..."
# Always reinstall basic backages
echo wms-interface-sqlite3 >> $pkgs_file
echo wildix-cw >> $pkgs_file
echo sipproxy >> $pkgs_file

pkgs=`sort $pkgs_file | uniq | sed ':a;N;$!ba;s/\n/ /g'`
(sudo /etc/init.d/ntp stop && sudo /usr/sbin/ntpdate-debian && sudo /etc/init.d/ntp start && touch /dev/shm/filetouch) >/dev/null
sudo -u wms /usr/sbin/wms_remote_support
if [ "x$?" != "x0" ] ; then
	sleep 10
	sudo -u wms /usr/sbin/wms_remote_support
fi
/usr/bin/apt-get clean && /usr/bin/apt-get update && /usr/bin/apt-get install --reinstall -y --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf $pkgs 2> $err_file
if [ "x$?" != "x0" ] ; then
        echo -ne "Re-installation of basic packages failed. Trying to fix..."
	apt-get -f --force-yes --allow-unauthenticated -q -c /rw2/etc/apt.conf install
	if [ "x$?" != "x0" ] ; then
		echo "failed!"
	else
		echo "done."
	fi
else
	echo "done."
	for i in $pkgs ; do
		if [ -f  "/var/lib/dpkg/info/${i}.conffiles" ] ; then
			while read j ; do
				if [ ! -f "$j" ] ; then
					if [ ! -d /tmp/package ] ; then
						mkdir -p /tmp/package
						dpkg -x /var/cache/apt/archives/$i*.deb /tmp/package
					fi
				cp /tmp/package${j} $j
				echo "Recovered conffile $j using default from package."
				fi			
			done < /var/lib/dpkg/info/${i}.conffiles
			rm -rf /tmp/package
		fi
	done
	if [ "x`ls -l /rw2/var/lib/ldap/ | grep Input`" != "x" ] ; then
		/usr/sbin/ldap_reset reset
	fi
/usr/bin/dpkg --configure --force-confold -a &> /dev/null
	echo "Your system is OK now."
fi
#Clean-up
rm -f $tmpfile
rm -f $pkgs_file
rm -f $err_file
